/*
 * Copyright 2014 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.depot;

import java.io.File;
import java.rmi.RemoteException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Locale;

import de.frankmuenster.jameica.depot.db.rmi.JaDepotDBService;
import de.frankmuenster.jameica.depot.db.server.JaDepotDBServiceImpl;
import de.frankmuenster.jameica.depot.gui.navigation.AktienKontoExtension;
import de.willuhn.datasource.db.EmbeddedDatabase;
import de.willuhn.jameica.gui.extension.ExtensionRegistry;
import de.willuhn.jameica.plugin.AbstractPlugin;
import de.willuhn.jameica.plugin.Manifest;
import de.willuhn.jameica.plugin.PluginResources;
import de.willuhn.jameica.plugin.Version;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Haupt-Klasse des jameica.qrscanner Plug-ins.
 * 
 * @author munstfr
 * @version 1.1
 * 
 */
public class JaDepot extends AbstractPlugin {

	/**
	 * Datums-Format dd.MM.yyyy HH:mm.
	 */
	public final static DateFormat LONGDATEFORMAT = new SimpleDateFormat(
			"dd.MM.yyyy HH:mm", Locale.GERMAN); //$NON-NLS-1$

	/**
	 * Datums-Format dd.MM.yyyy.
	 */
	//public final static DateFormat DATEFORMAT = new CustomDateFormat(
	//		"dd.MM.yyyy"); //$NON-NLS-1$

	/**
	 * Datums-Format ddMMyyyy.
	 */
	public final static DateFormat FASTDATEFORMAT = new SimpleDateFormat(
			"ddMMyyyy", Locale.GERMAN); //$NON-NLS-1$

	/**
	 * DecimalFormat.
	 */
	//public final static DecimalFormat DECIMALFORMAT = new CustomDecimalFormat();

	/**
	 * Standardkonstruktor
	 */
	public JaDepot() {
		super();
	}

	/**
	 * Methode die beim Starten des Plug-Ins geladen wird
	 * 
	 * @see de.willuhn.jameica.plugin.AbstractPlugin#init()
	 */
	@Override
	public void init() throws ApplicationException {
		super.init();

		Logger.info(Settings.getI18n().tr("JaDepot.startInit")); //$NON-NLS-1$

		// Datenbank Konsistenz berprfen
		call(new ServiceCall() {
			/**
			 * @param service
			 * @throws ApplicationException
			 * @throws RemoteException
			 */
			@Override
			public void call(final JaDepotDBService service)
					throws ApplicationException, RemoteException {
				service.checkConsistency();
			}
		});

		// Menu des Jameica-'Plugins' erweitern
		ExtensionRegistry.register(new AktienKontoExtension(), "jameica.start"); //$NON-NLS-1$

		Logger.info(Settings.getI18n().tr("JaDepot.endInit")); //$NON-NLS-1$
	}

	/**
	 * This method is called only the first time, the plugin is loaded (before
	 * executing init()). if your installation procedure was not successfull,
	 * throw an ApplicationException. You dont need to implement this function.
	 * 
	 * @see de.willuhn.jameica.plugin.AbstractPlugin#install()
	 */
	@Override
	public void install() throws ApplicationException {

		// If we are running in client/server mode and this instance
		// is the client, we do not need to create a database.
		// Instead of this we will get our objects via RMI from
		// the server
		if (Application.inClientMode()) {
			return;
		}

		try {

			// Let's create an embedded Database
			final PluginResources res = Application.getPluginLoader()
					.getPlugin(JaDepot.class).getResources();
			final Manifest manifest = Application.getPluginLoader()
					.getManifest(JaDepot.class);
			final EmbeddedDatabase database = new EmbeddedDatabase(
					res.getWorkPath() + "/db", "aktien", "aktview"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

			// create the sql tables.
			database.executeSQLScript(new File(manifest.getPluginDir()
					+ File.separator + "sql", "create.sql")); //$NON-NLS-1$ //$NON-NLS-2$

			// That's all. Database installed and tables created ;)
		} catch (Exception e) { // NOPMD
			throw new ApplicationException(Settings.getI18n().tr(
					"JaDepot.errorInstalling"), e); //$NON-NLS-1$
		}
	}

	/**
	 * This method will be executed on every version change. You don't need to
	 * implement this function.
	 * 
	 * @see de.willuhn.jameica.plugin.AbstractPlugin#update(de.willuhn.jameica.plugin.Version)
	 */
	@Override
	public void update(final Version oldVersion) throws ApplicationException {
		super.update(oldVersion);
		Logger.trace(Settings.getI18n().tr("JaDepot.inUpdate")); //$NON-NLS-1$
	}

	/**
	 * Here you can do some cleanup stuff. The method will be called on every
	 * clean shutdown of jameica. You dont need to implement this function.
	 * 
	 * @see de.willuhn.jameica.plugin.AbstractPlugin#shutDown()
	 */
	@Override
	public void shutDown() {
		super.shutDown();
		Logger.trace(Settings.getI18n().tr("JaDepot.inShutdown")); //$NON-NLS-1$
	}

	/**
	 * Hilfsmethode zum bequemen Ausfuehren von Aufrufen auf dem Service.
	 */
	private interface ServiceCall {
		/**
		 * @param service
		 * @throws ApplicationException
		 * @throws RemoteException
		 */
		void call(JaDepotDBService service) throws ApplicationException,
				RemoteException;
	}

	/**
	 * Hilfsmethode zum bequemen Ausfuehren von Methoden auf dem Service.
	 * 
	 * @param call
	 *            der Call.
	 * @throws ApplicationException
	 */
	private void call(final ServiceCall call) throws ApplicationException {
		if (Application.inClientMode()) {
			return; // als Client muessen wir die DB nicht installieren
		}

		JaDepotDBService service = null;
		try {
			// Da die Service-Factory zu diesem Zeitpunkt noch nicht da ist,
			// erzeugen
			// wir uns eine lokale Instanz des Services.
			service = new JaDepotDBServiceImpl();
			service.start();
			call.call(service);
		} catch (RemoteException e) {
			Logger.error(Settings.getI18n().tr("JaDepot.errorInitDb"), e); //$NON-NLS-1$
			// final I18N i18n = getResources().getI18N();
			final StringBuffer msg = new StringBuffer(491);
			msg.append(Settings.getI18n().tr("JaDepot.errorInitDbMsg", //$NON-NLS-1$
					e.getMessage()));
			throw new ApplicationException(msg.toString(), e);
		} finally {
			if (service != null) {
				try {
					service.stop(true);
				} catch (RemoteException e) {
					Logger.error(
							Settings.getI18n().tr("JaDepot.errorCloseDb"), e); //$NON-NLS-1$
				}
			}
		}
	}
}

/***********************************************************
 * History: $Log AktienKonto.java,v$ $Date:: 2014-04-01 00:56:03$ Date of last
 * commit
 * 
 * @Revision 1.1 Date: 20.04.2014 Navigationmen umgestellt von plugins-xml zu
 *           Extension ExtensionRegistry.register hinzugefgt
 * 
 *********************************************************/
