/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.depot.gui.parts;

import java.rmi.RemoteException;
import java.sql.Time;
import java.util.Date;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.gui.controller.WertpapierDetailController;
import de.frankmuenster.jameica.depot.gui.input.LabelInputExtended;
import de.frankmuenster.jameica.depot.gui.utils.DateUtils;
import de.frankmuenster.jameica.depot.gui.utils.FormatterUtils;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.util.Color;
import de.willuhn.jameica.gui.util.Container;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;

/**
 * Lieft ein Part, das details zu einem Wertpapier anzeigt. Dazu geh&ouml;ren
 * der Name, das Yahoo-Symbol, die Anzahl, das letzte Abfragedatum und der Kurs,
 * sowie die Gesamtsumme des Papieres im Depot. Ist das Papier nicht bei Yahoo
 * gelistet, knnen Datum, Kurs und W&auml;hrung gesetzt werden.
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public class WertpapierDetailsPart implements Part {

	/**
	 * Veraltertes Handelsdatum in Millisekunden.
	 */
	private final static long OLD_TRADE_DAYS = 7 * 24 * 60 * 60 * 1000;

	/**
	 * Veraltertes Handelsdatum Anzeigentext
	 */
	private final static String OLD_TRADE_TEXT = Settings.getI18n().tr(
			"WertpapierDetailsPart.toOldLastTrade"); //$NON-NLS-1$

	/**
	 * Pfeil nach oben Zeichen
	 */
	private final static char ARROW_UP = '\u2191';

	/**
	 * Pfeil nach unten Zeichen
	 */
	private final static char ARROW_DOWN = '\u2193';

	/**
	 * WertpapierDetail-Kontroller, damit auch Eingabefelder f verwendet werden
	 * k&ouml;nnen.
	 */
	private final transient WertpapierDetailController control;

	/**
	 * Das Wertpapier
	 */
	private final transient Wertpapier wertpapier;

	/**
	 * Der WP-Name
	 */
	private transient LabelInputExtended nameLbl;

	/**
	 * Das WP-Symbol
	 */
	private transient LabelInputExtended symbolLbl;

	/**
	 * Die WP-St&uuml;ckanzahl
	 */
	private transient LabelInputExtended anzahlLbl;

	/**
	 * Das letzte Handelsdatum
	 */
	private transient LabelInputExtended datumLbl;

	/**
	 * Der letzte Handelswert
	 */
	private transient LabelInputExtended letzterWertLbl;

	/**
	 * Der Gesamtwert
	 */
	private transient LabelInputExtended totalLbl;

	/**
	 * Erstellt ein neues WertpapierDetailPart.
	 * 
	 * @param wertpapier
	 *            das Papier dessen Details angezeigt werden sollen.
	 * @param control
	 *            der Kontroller, der
	 */
	public WertpapierDetailsPart(final Wertpapier wertpapier,
			final WertpapierDetailController control) {
		this.control = control;
		this.wertpapier = wertpapier;

	}

	/**
	 * @see Part#paint(Composite)
	 */
	@Override
	public void paint(final Composite parent) throws RemoteException {

		final SimpleContainer container = new SimpleContainer(parent, false, 6);

		if (wertpapier == null) {
			container.addText(
					Settings.getI18n().tr(
							"WertpapierDetailsPart.noStockInfoAvailable"), //$NON-NLS-1$
					true);
		} else {
			nameLbl = new LabelInputExtended(wertpapier.getName());
			container.addLabelPair(

			Settings.getI18n().tr("WertpapierDetailsPart.lblName"), nameLbl); //$NON-NLS-1$

			symbolLbl = new LabelInputExtended(wertpapier.getSymbol());
			container
					.addLabelPair(
							Settings.getI18n().tr(
									"WertpapierDetailsPart.lblSymbole"), symbolLbl); //$NON-NLS-1$

			final double anzahl = wertpapier.getAnzahl();
			anzahlLbl = new LabelInputExtended(FormatterUtils.DECIMAL_FORMAT_6
					.get().format(anzahl), Settings.getI18n().tr(
					"WertpapierDetailsPart.lblNumberStocks")); //$NON-NLS-1$
			container
					.addLabelPair(
							Settings.getI18n().tr(
									"WertpapierDetailsPart.lblCount"), anzahlLbl); //$NON-NLS-1$

			if (wertpapier.isInYahoo()) {
				wpInYahoo(container);
			} else {
				wpNotInYahoo(parent);
			}

		}

	}

	/**
	 * Aktualisiert die Anzeige.
	 */
	public void update() {

		try {

			nameLbl.setValue(wertpapier.getName());
			symbolLbl.setValue(wertpapier.getSymbol());
			anzahlLbl.setValue(FormatterUtils.DECIMAL_FORMAT_6.get().format(
					wertpapier.getAnzahl()));

			// Datum und LetzerHandelsWert nur wenn nicht als Inputs
			if (datumLbl != null) {
				datumLbl.setValue(FormatterUtils.GERMAN_DATE_FORMAT.get()
						.format(wertpapier.getLastTradeDate()));
				final Date date = wertpapier.getLastTradeDate();
				final Time time = wertpapier.getLastTradeTime();

				if (DateUtils.dateDiff(date) > OLD_TRADE_DAYS) {
					datumLbl.setColor(Color.ERROR);
					datumLbl.setToolTip(Settings.getI18n().tr(OLD_TRADE_TEXT));
					Application.getMessagingFactory()
							.sendMessage(
									new StatusBarMessage(Settings.getI18n().tr(
											OLD_TRADE_TEXT),
											StatusBarMessage.TYPE_INFO));
				} else {
					datumLbl.setColor(Color.FOREGROUND);
					if (time != null) {
						datumLbl.setToolTip(FormatterUtils.SIMPLE_TIME_FORMAT
								.get().format(time));
					}
				}
			}
			if (letzterWertLbl != null) {
				letzterWertLbl.setValue(FormatterUtils.DECIMAL_FORMAT_3.get()
						.format(wertpapier.getLastTradePriceOnly()) + " " //$NON-NLS-1$
						+ wertpapier.getCurrency());
			}

			final StringBuffer change = new StringBuffer(
					FormatterUtils.DECIMAL_FORMAT.get().format(
							wertpapier.getChangeInPercentage()));
			if (wertpapier.getChangeInPercentage() < 0) {
				change.append(ARROW_DOWN).append(change);
				letzterWertLbl.setColor(Color.ERROR);
			} else {
				change.append(ARROW_UP).append(change);
				letzterWertLbl.setColor(Color.FOREGROUND);
			}
			letzterWertLbl.setComment(change + "%"); //$NON-NLS-1$
			letzterWertLbl.setToolTip(Settings.getI18n().tr(
					"WertpapierDetailsPart.lblValueTooltip")); //$NON-NLS-1$

			totalLbl.setValue(FormatterUtils.CURRENCY_FORMAT.get()
					.format(wertpapier.getAnzahl()
							* wertpapier.getLastTradePriceOnly())
					+ " " + wertpapier.getCurrency()); //$NON-NLS-1$
		} catch (RemoteException e) {
			Logger.error(Settings.getI18n().tr(
					"WertpapierDetailsPart.refreshStockDeatilPartError") //$NON-NLS-1$
					+ e.getMessage());
			Application
					.getMessagingFactory()
					.sendMessage(
							new StatusBarMessage(
									Settings.getI18n()
											.tr("WertpapierDetailsPart.refreshStockDeatilPartErrorMsg"), //$NON-NLS-1$
									StatusBarMessage.TYPE_ERROR));
		}
	}

	/**
	 * Zeichnet die WP Daten aus Yahoo.
	 * 
	 * @param container
	 *            der schon erstellte Container.
	 * @throws RemoteException
	 */
	private void wpInYahoo(final Container container) throws RemoteException {
		final Date date = wertpapier.getLastTradeDate();
		final Time time = wertpapier.getLastTradeTime();
		if (date == null) {
			datumLbl = new LabelInputExtended(StringUtils.EMPTY, Settings
					.getI18n().tr("WertpapierDetailsPart.lblDatePleaseRefesh")); //$NON-NLS-1$
		} else {
			datumLbl = new LabelInputExtended(FormatterUtils.SIMPLE_TIME_FORMAT
					.get().format(date));
			// long dateDiff = 7 * 24 * 60 * 60 * 1000;
			if (DateUtils.dateDiff(date) > OLD_TRADE_DAYS) {
				datumLbl.setColor(Color.ERROR);
				datumLbl.setToolTip(Settings.getI18n().tr(OLD_TRADE_TEXT));
				Application.getMessagingFactory().sendMessage(
						new StatusBarMessage(Settings.getI18n().tr(
								OLD_TRADE_TEXT), StatusBarMessage.TYPE_INFO));
			} else {
				// Uhrzeit anzeigen
				if (time != null) {
					datumLbl.setToolTip(FormatterUtils.SIMPLE_TIME_FORMAT.get()
							.format(time));
				}
			}
		}
		container
				.addLabelPair(
						Settings.getI18n().tr("WertpapierDetailsPart.lblDate"), datumLbl); //$NON-NLS-1$

		final double lastTradePrice = wertpapier.getLastTradePriceOnly();
		letzterWertLbl = new LabelInputExtended(FormatterUtils.DECIMAL_FORMAT_3
				.get().format(lastTradePrice) + " " + wertpapier.getCurrency()); //$NON-NLS-1$

		// Platz machen fr die nderungsanzeige
		letzterWertLbl.setComment(StringUtils.EMPTY);

		container
				.addLabelPair(
						Settings.getI18n().tr("WertpapierDetailsPart.lblValue"), letzterWertLbl); //$NON-NLS-1$

		final double anzahl = wertpapier.getAnzahl()
				* wertpapier.getLastTradePriceOnly();
		totalLbl = new LabelInputExtended(FormatterUtils.CURRENCY_FORMAT.get()
				.format(anzahl) + " " + wertpapier.getCurrency(), Settings //$NON-NLS-1$
				.getI18n().tr("WertpapierDetailsPart.lblTotalTooltip")); //$NON-NLS-1$
		container
				.addLabelPair(
						Settings.getI18n().tr("WertpapierDetailsPart.lblTotal"), totalLbl); //$NON-NLS-1$

	}

	/**
	 * Zeichnet die WP Daten aus Yahoo.
	 * 
	 * @param parent
	 * @throws RemoteException
	 */
	private void wpNotInYahoo(final Composite parent) throws RemoteException {
		final SimpleContainer notYahoo = new SimpleContainer(parent, false, 6);

		notYahoo.addLabelPair(
				Settings.getI18n().tr("WertpapierDetailsPart.lblDate"), //$NON-NLS-1$
				control.getDatum());
		notYahoo.addLabelPair(
				Settings.getI18n().tr("WertpapierDetailsPart.lblValue"), //$NON-NLS-1$
				control.getLetzterKursWert());
		notYahoo.addPart(control.getWaehrung());

		final double totalValue = wertpapier.getAnzahl()
				* wertpapier.getLastTradePriceOnly();
		totalLbl = new LabelInputExtended(FormatterUtils.CURRENCY_FORMAT.get()
				.format(totalValue) + " " + wertpapier.getCurrency(), Settings //$NON-NLS-1$
				.getI18n().tr("WertpapierDetailsPart.lblTotalTooltip")); //$NON-NLS-1$
		notYahoo.addLabelPair(
				Settings.getI18n().tr("WertpapierDetailsPart.lblTotal"), totalLbl); //$NON-NLS-1$

		final StringBuffer message = new StringBuffer(Settings.getI18n().tr(
				"WertpapierDetailsPart.quoteNotinYahoo")); //$NON-NLS-1$

		if (control.getDatum().getValue() != null
				&& DateUtils.dateDiff((Date) control.getDatum().getValue()) > OLD_TRADE_DAYS) {
			message.append(" | ").append(Settings.getI18n().tr(OLD_TRADE_TEXT)); //$NON-NLS-1$
		}

		Application.getMessagingFactory().sendMessage(
				new StatusBarMessage(message.toString(),
						StatusBarMessage.TYPE_INFO));
	}

}
