/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.depot.gui.parts.column;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.CurrencyConverterYahoo;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.YahooFinanceException;
import de.frankmuenster.jameica.depot.gui.utils.FormatterUtils;
import de.willuhn.jameica.gui.formatter.Formatter;
import de.willuhn.jameica.gui.parts.Column;
import de.willuhn.logging.Logger;

/**
 * Klasse die eine Spalte mit Betr&auml;gen in der Standartw&auml;hrung
 * erstellt. Dabei kann entweder das aktuelle Datum oder das Datum der Order
 * genommen werden.
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public abstract class AbstractColumnInStandardWaehrung extends Column {

	/**
	 * Unser serial UID
	 */
	private static final long serialVersionUID = -5340194795194610509L;

	/**
	 * Eine Liste mit Umtauschraten der Wertpapiere, die nicht in der
	 * Standardw&auml;hrung sind.
	 */
	private final transient List<ExchangeInfo> exchangeInfos = new ArrayList<ExchangeInfo>();

	/**
	 * Soll das Datum der Order benutzt werden, um die Umtauschrate zu
	 * ermitteln. Falls nein, wird die aktuellste Umtauschrate verwendet.
	 */
	private transient boolean useOrderDate;

	/**
	 * Der W&auml;hrungsumrechner
	 */
	private final transient CurrencyConverterYahoo ccy = CurrencyConverterYahoo
			.getInstance();

	/**
	 * Konstruktor mit Angabe eines Formatters.
	 * 
	 * @param formatter
	 */
	public AbstractColumnInStandardWaehrung(final Formatter formatter) {
		this(false, formatter);
	}

	/**
	 * Konstruktor mit Angabe eines Formatters und dem useOrderDate Value. Mit
	 * der oderDate kann angegeben werden, ob die Umtauschrate zum Zeitpunkt der
	 * Order oder die aktuellste zur Verf&uuml;gung stehendes angezeigt werden
	 * soll.
	 * 
	 * @param useOrderDate
	 * @param formatter
	 */
	public AbstractColumnInStandardWaehrung(final boolean useOrderDate,
			final Formatter formatter) {
		super("id", getStandardWaehrung(), formatter, false, ALIGN_RIGHT); //$NON-NLS-1$
		this.useOrderDate = useOrderDate;
	}

	/**
	 * Liefert eine List emit allen ermittelten Umtauschraten.
	 * 
	 * @return Liste mit Umtauschraten.
	 */
	public List<ExchangeInfo> getExchangeInfos() {
		return exchangeInfos;
	}

	/**
	 * Liefert den formatieren Zeileneintrag.
	 * 
	 * @param betrag
	 * @param waehrung
	 * @param date
	 * @return den formatieren Zeileneintrag
	 */
	protected String getFormattedValue(final double betrag,
			final String waehrung, final Date date) {
		String display = StringUtils.EMPTY;
		final String defaultWaehrung = Settings.getDepotWaehrung();

		if (waehrung != null && !waehrung.equals(defaultWaehrung)) {

			try {
				double changeRate;
				if (date == null) {
					changeRate = ccy.exchangeRate(waehrung, defaultWaehrung);
				} else {
					changeRate = ccy.exchangeRate(waehrung, defaultWaehrung,
							date);
				}
				final ExchangeInfo exchangeInfo = new ExchangeInfo(waehrung,
						defaultWaehrung, date, changeRate);
				storeExchange(exchangeInfo);

				final double value = betrag * changeRate;

				// Formatter definiert?
				if (this.getFormatter() == null) {
					display = FormatterUtils.CURRENCY_FORMAT.get()
							.format(value) + defaultWaehrung;
				} else {
					display = this.getFormatter().format(value);
				}

			} catch (YahooFinanceException e) {
				Logger.error(
						Settings.getI18n()
								.tr(Settings
										.getI18n()
										.tr("AbstractColumnInStandardWaehrung.calculatingError")), //$NON-NLS-1$
						e);
			}
		}
		return display;
	}

	/**
	 * F&uuml;gt eine Umtauschrate zur Liste hinzu und vermeidet Dopplungen.
	 * 
	 * @param exchangeInfo
	 */
	private void storeExchange(final ExchangeInfo exchangeInfo) {

		for (final ExchangeInfo eis : exchangeInfos) {
			if (eis.getWaehrungFrom().equals(exchangeInfo.getWaehrungFrom())
					&& eis.getWaehrungTo().equals(exchangeInfo.getWaehrungTo())) {
				// Schon in Liste --> aktualisieren
				eis.setDatum(exchangeInfo.getDatum());
				eis.setKurs(exchangeInfo.getKurs());
				return;
			}
		}

		// Nicht in Liste --> hinzufgen
		exchangeInfos.add(exchangeInfo);

	}

	/**
	 * Kleine Funktion die den Titel ein f&uuml;r Kurswertspalte mit W&ahrung
	 * erzeugt.
	 * 
	 * @return den Splatentitel
	 */
	private static String getStandardWaehrung() {
		return Settings.getI18n().tr("AbstractColumnInStandardWaehrung.in") + Settings.getDepotWaehrung(); //$NON-NLS-1$
	}

	/**
	 * Wird Orderdatum zur Anzeige der Umtauschrate verwendet?.
	 * 
	 * @return useOrderDate
	 */
	protected boolean isUseOrderDate() {
		return this.useOrderDate;
	}
}
