/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.depot.gui.parts.grafiken;

import java.awt.Font;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.jfree.chart.JFreeChart;
import org.jfree.data.time.Day;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.data.xy.DefaultOHLCDataset;
import org.jfree.data.xy.OHLCDataItem;
import org.jfree.data.xy.OHLCDataset;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.HistoryValueBean;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.WertpapierHistory;
import de.willuhn.jameica.gui.Part;

/**
 * Abstrakte Klasse zur Anzeige des Charts eines Wertpapieres. Als Chart wird
 * JFrreeChart verwendet.
 * 
 * @author munstfr
 * @version 1.0
 */
public abstract class AbstractGrafik implements Part {

	/**
	 * Formatter f&uuml;r die Kurswert-Annotation
	 */
	protected final static DecimalFormat DECIMAL_FOMAT = new DecimalFormat("  ###,##0.00"); //$NON-NLS-1$
	
	/**
	 * Achsenbeschriftung Font
	 */
	protected final static Font FONT_AXIS = new Font("Dialog", Font.PLAIN, 12); //$NON-NLS-1$

	/**
	 * Der Titel im JFreeChart
	 */
	protected String title;

	/**
	 * Das JFreeChart das unten drunter benutzt wird.
	 */
	protected transient JFreeChart chart;

	/**
	 * Anfangsdatum
	 */
	protected Date fromDate;

	/**
	 * Enddatum
	 */
	protected Date toDate;

	/**
	 * Die Werte des Wertpapieres
	 */
	protected transient WertpapierHistory wertpapierHistory;

	/**
	 * Die DAX-Werte
	 */
	protected transient WertpapierHistory daxHistory;

	/**
	 * Zeige DAX im CHART
	 */
	protected boolean showDax;

	/**
	 * Zeige Chart als Kerzengrafik
	 */
	protected transient boolean showCandle;

	/**
	 * Standardkonstruktor
	 */
	public AbstractGrafik() {
		// Absichtlich leer
	}

	/**
	 * Liefert den Titel
	 * 
	 * @return der Charttitel
	 */
	public String getTitle() {
		return this.title;
	}

	/**
	 * Setzt den Titel des Charts
	 * 
	 * @param title
	 */
	public void setTitle(final String title) {
		this.title = title;
		if (this.chart != null) {
			this.chart.setTitle(this.title);
		}
	}

	/**
	 * @return fromDate
	 */
	public Date getFromDate() {
		return fromDate;
	}

	/**
	 * @param fromDate
	 *            das zu setzende Objekt fromDate
	 */
	public void setFromDate(final Date fromDate) {
		this.setRange(fromDate, this.toDate);
	}

	/**
	 * @return toDate
	 */
	public Date getToDate() {
		return this.toDate;
	}

	/**
	 * @param toDate
	 *            das zu setzende Objekt toDate
	 */
	public void setToDate(final Date toDate) {
		this.setRange(this.fromDate, toDate);
	}

	/**
	 * Setzt den Zeitraum, der angezeigt werden soll.
	 * 
	 * @param fromDate
	 *            das Anfangsdatum.
	 * @param toDate
	 *            das Enddatum.
	 */
	public void setRange(final Date fromDate, final Date toDate) {
		this.fromDate = fromDate;
		this.toDate = toDate;
	}

	/**
	 * @return showDax
	 */
	public boolean isShowDax() {
		return showDax;
	}

	/**
	 * @param showDax
	 *            das zu setzende Objekt showDax
	 */
	public void setShowDax(final boolean showDax) {
		this.showDax = showDax;
		if (this.showDax) {
			this.showCandle = false; // Beides geht nicht!
		}
	}

	/**
	 * @return the showCandle
	 */
	public boolean isShowCandle() {
		return showCandle;
	}

	/**
	 * @param showCandle
	 *            the showCandle to set
	 */
	public void setShowAsCandleStick(final boolean showCandle) {
		this.showCandle = showCandle;
		if (this.showCandle) {
			this.showDax = false; // Beides geht nicht!!!
		}
	}

	/**
	 * @param wertpapierHistory
	 *            the wertpapierHistory to set
	 * @param daxHistory
	 * @throws IllegalArgumentException
	 */
	public void setKursDaxDaten(final WertpapierHistory wertpapierHistory,
			final WertpapierHistory daxHistory) throws IllegalArgumentException {

		if (wertpapierHistory == null || daxHistory == null) {
			throw new IllegalArgumentException(Settings.getI18n().tr(
					"AbstractGrafik.nullArgument")); //$NON-NLS-1$
		}

		this.wertpapierHistory = wertpapierHistory;
		this.daxHistory = daxHistory;
	}

	/**
	 * Neuladen den Grafik.
	 */
	public abstract void reload();

	/**
	 * Lieft das JFreeChart Objekt.
	 * 
	 * @return das JFreeChart Objekt
	 */
	public JFreeChart getChart() {

		return chart;
	}

	/**
	 * Liefert eine JFreeChart TimeSeriesCollection der Wertpapierdaten.
	 * 
	 * @return JFreeChart TimeSeriesCollection der Wertpapierdaten
	 */
	protected TimeSeriesCollection getDataSetKurs() {

		final TimeSeries seriesKurs = new TimeSeries(Settings.getI18n().tr(
				"AbstractGrafik.timeSeriesRate")); //$NON-NLS-1$

		final List<HistoryValueBean> historyValueBean = wertpapierHistory
				.getTimeRangeValues(fromDate, toDate);

		for (final HistoryValueBean wphv : historyValueBean) {

			final Date date = wphv.getDate();
			final double close = wphv.getClose();

			// Neues Datumsobjekt erstellen (wirklich ntig?)
			Day day = new Day(date); // NOPMD
			seriesKurs.add(day, close);
		}

		final TimeSeriesCollection collection = new TimeSeriesCollection();
		collection.addSeries(seriesKurs);
		return collection;
	}

	/**
	 * Liefert eine JFreeChart TimeSeriesCollection den prozentualen
	 * Ver&auml;nderungen des Wertpapier und DAX Kurses.
	 * 
	 * @return JFreeChart TimeSeriesCollection der Wertpapierdaten
	 */
	protected TimeSeriesCollection getDatasetKursDaxVergleich() {

		final TimeSeries seriesKurs = new TimeSeries(Settings.getI18n().tr(
				"AbstractGrafik.timeSeriesRate")); //$NON-NLS-1$
		final TimeSeries seriesDax = new TimeSeries("DAX"); //$NON-NLS-1$

		final List<HistoryValueBean> historyValueBean = wertpapierHistory
				.getTimeRangeValues(fromDate, toDate);
		final List<HistoryValueBean> daxHistoryValues = daxHistory
				.getTimeRangeValues(fromDate, toDate);

		final double kursStart = historyValueBean.get(0).getClose();
		final double daxStart = daxHistoryValues.get(0).getClose();

		for (final HistoryValueBean dax : daxHistoryValues) {

			final Date date = dax.getDate();
			final double daxValue = dax.getClose();
			final double closeDax = ((daxValue - daxStart) * 100) / daxStart;

			final Day day = new Day(date); // NOPMD
			seriesDax.add(day, closeDax);
		}

		for (final HistoryValueBean kurs : historyValueBean) {

			final Date date = kurs.getDate();
			final double kursValue = kurs.getClose();
			final double closeKurs = ((kursValue - kursStart) * 100)
					/ kursStart;

			final Day day = new Day(date); // NOPMD
			seriesKurs.add(day, closeKurs);
		}

		final TimeSeriesCollection collection = new TimeSeriesCollection();
		collection.addSeries(seriesKurs);
		collection.addSeries(seriesDax);
		return collection;
	}

	/**
	 * Liefert ein JFreeChart OHLCDataset. Beinhaltet alle ntigen Daten fr
	 * eine Kerzengrafik. Dazu geh&ouml;ren u.a. Er&ouml;ffnungs, Schlusskurs,
	 * Tageshoch-tief, etc.
	 * 
	 * @return {@link OHLCDataset} mit Kerzengrafikdaten.
	 */
	protected OHLCDataset getDatasetCandleStick() {

		final List<OHLCDataItem> dataItems = new ArrayList<OHLCDataItem>();
		final List<HistoryValueBean> historyValueBean = wertpapierHistory
				.getTimeRangeValues(fromDate, toDate);

		for (final HistoryValueBean wphv : historyValueBean) {

			final Date date = wphv.getDate();
			final double open = wphv.getOpen();
			final double high = wphv.getHigh();
			final double low = wphv.getLow();
			final double close = wphv.getClose();
			final double volume = wphv.getVolume();
			wphv.getAdjClose();

			final OHLCDataItem item = new OHLCDataItem(date, open, high, low, // NOPMD
					close, volume);
			dataItems.add(item);
		}

		final OHLCDataItem[] data = dataItems
				.toArray(new OHLCDataItem[dataItems.size()]);
		final OHLCDataset dataset = new DefaultOHLCDataset(
				wertpapierHistory.getSymbol(), data);

		return dataset;
	}

}